/*
 * clock for root window.
 */

/*
 * - finish alarms set up
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <math.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <locale.h>
#include <string.h>
#include <strings.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include "config.h"
#include "options.h"
#include "rootapp.h"

#define EVENTMASK VisibilityChangeMask | ExposureMask

#define pi 3.1415926


/* Global varriables */
static int end_program = FALSE;

static char      *dpyname    = NULL;
static char      *geometry   = NULL;
static char      *mcolorname = NULL;
static char      *scolorname = NULL;
static int        radius     = 10;
static int        h, m, s;
static unsigned long cm = 0, cs = 0;


/* Command line options and config file entries */
static Option opttab[] = {
    {'h', NULL,      NULL, O_HELP,   NULL,        FALSE, "display help"},
    {'v', NULL,      NULL, O_VERS,   NULL,        FALSE, "display version"},
    {'d', "display", NULL, C_STRING, &dpyname,    FALSE, "display name"},
    {'g', "geom",    NULL, C_STRING, &geometry,   FALSE, "set geometry"},
    {'c', "color",   NULL, C_STRING, &mcolorname, FALSE, "main color"},
    {'s', "color",   NULL, C_STRING, &scolorname, FALSE, "seconds color"},
    {'r', "radius",  NULL, C_INT,    &radius,     FALSE, "clock radius"},
    {0,   NULL,      NULL, C_NONE,   NULL,        FALSE, NULL}
};



/****** UPDATING FUNCTIONS ******/

void update_now() {
    time_t     tnow;
    struct tm *now;

    time(&tnow);
    now = localtime(&tnow);
    h = now->tm_hour;
    m = now->tm_min;
    s = now->tm_sec;
}


void calc_point(int position, Bool is_hour, int length, int *x, int *y) {
    int a;

    if (is_hour)
        a = 30 * (position - 3);
    else
        a = 6 * (position - 15);

    *x = radius + length * cos(a*pi/180);
    *y = radius + length * sin(a*pi/180);
}


void set_default_segments(XSegment segments[]) {
    int i;

    /* All */
    for (i = 0 ; i < 12 ; i++) {
        int x, y;
        calc_point(i, True, radius, &x, &y);
        segments[i].x1 = x;
        segments[i].y1 = y;
        segments[i].x2 = x;
        segments[i].y2 = y;
    }
}


void draw_time(RootApp *app, XSegment segments[]) {
    int x, y;

    /* Hours */
    calc_point(h, True, radius / 2, &x, &y);
    segments[12].x1 = radius;
    segments[12].y1 = radius;
    segments[12].x2 = x;
    segments[12].y2 = y;

    /* Minutes */
    calc_point(m, False, 2 * radius / 3, &x, &y);
    segments[13].x1 = radius;
    segments[13].y1 = radius;
    segments[13].x2 = x;
    segments[13].y2 = y;

    set_color(cm);
    /* Markers, hours' hand, minutes' hand */
    XDrawSegments(display, app->bgpixmap, gc, segments, 14);

    /* seconds */
    set_color(cs);
    calc_point(s, False, 3 * radius / 4, &x, &y);
    XDrawLine(display, app->bgpixmap, gc, radius, radius, x, y);
}


/****** PROCESSING FUNCTIONS ******/

void main_loop(RootApp *app) {
    XSegment segments[14];
    XEvent   event;
    int      cont = TRUE;

    set_default_segments(segments);
    draw_time(app, segments);
    display_app(app);
    while (cont) {
        nextevent_or_timeout(&event, 1000);
        update_now();
        clear_bg(app);
        draw_time(app, segments);
        display_app(app);
        if (end_program) cont = FALSE;
    }
}



/****** SIGNALS ******/

void install_handler();


void signals_handler(int sig) {
    switch (sig) {
        case SIGINT:
        case SIGTERM:
        case SIGQUIT:
            end_program = TRUE;
            break;
    }
}


void install_handler() {
    signal(SIGTERM, signals_handler);
    signal(SIGQUIT, signals_handler);
    signal(SIGINT,  signals_handler);
}


void manage_signals() {
    struct sigaction sa;

    sa.sa_handler = SIG_IGN;
#ifdef SA_NOCLDWAIT
    sa.sa_flags = SA_NOCLDWAIT;
#else
    sa.sa_flags = 0;
#endif
    sigemptyset(&sa.sa_mask);
    sigaction(SIGCHLD, &sa, NULL);
    install_handler();
}


/****** MAIN ******/


int main(int argc, char *argv[]) {
    RootApp    *app;

    manage_signals();
    setlocale(LC_ALL, "");
    parse_argv(opttab, argc, argv);

    if (radius < 10) radius = 10;
    update_now();

    /* X11 init */
    open_x11(dpyname);
    if (mcolorname) cm = get_color(mcolorname);
    if (scolorname) cs = get_color(scolorname);

    /* Creation */
    app = create_app(EVENTMASK, geometry);
    resize(app, geometry, 0, 0, 2 * radius + 1, 2 * radius + 1);

    main_loop(app);

    /* Clean */
    close_app(app);
    close_x11();
    free_vars(opttab);

    return 0;
}
